@@ -17,8 +17,5 @@ systrayNew = do
     widgetShowAll w
     boxPackStart box w PackNatural 0
 
-  _ <- on trayManager trayIconRemoved $ \_ -> do
-    putStrLn "Tray icon removed"
-
   widgetShowAll box
-  return (toWidget box)
\ No newline at end of file
+  return (toWidget box)
